# 默认配置路径
import os
import shutil
import socket
import time

import click
import pexpect
import psutil

from src.config.global_config import private_key_default_file, compile_ip, compile_host_mame, \
    compile_tun_value
from src.server import server_store
from src.server.server_store import ServerConfig


def server_add(name, host, port, username, password, path):
    """
    添加服务配置
    :param name: 服务器名称
    :param host: 服务器地址
    :param port: 服务器ssh端口
    :param username: 服务器用户名
    :param password: 服务器用户密码
    :param path: 文件地址
    :return:
    """
    name = name.strip()
    secret_key_path = None
    if path is not None:
        abs_path = path
        # 处理用户目录
        if str(path).startswith('~'):
            abs_path = str(path).replace('~', os.path.expanduser('~'))
        click.echo(abs_path)
        if os.path.exists(abs_path) and os.path.isfile(abs_path):
            # 计算密钥地址
            secret_key_path = private_key_default_file + '/{}_{}.id_rsa'.format(name, username)
            click.echo(abs_path + '00' + secret_key_path)
            # 复制密钥到配置目录命名规则 服务器名_用户名_host地址
            shutil.copyfile(abs_path, secret_key_path)
        else:
            raise click.FileError('保存密钥异常！未找到密钥或者无权限')

    sc = ServerConfig(name=name, host=host, port=port, username=username, password=password,
                      secret_key_path=secret_key_path)
    server_store.create(sc)
    echo_str = '\n录入的服务器信息:\n名称:{}\n地址:{}\nssh端口:{}'.format(name, host, port)
    if secret_key_path is not None:
        echo_str += '\n密钥地址:{}'.format(secret_key_path)
    click.echo(echo_str)


def server_edit():
    server_store.edit()


def server_remove(name):
    """
    删除服务器配置信息
    获取所有配置  删除其中name符合的数据
    :param name: 服务器名称
    :return:
    """
    server_store.remove_by_name(name)


def server_list():
    sc_list = server_store.find_all()
    if sc_list is None:
        return
    config_str = "服务器信息列表:\n"
    for index, sc in enumerate(sc_list):
        config_str += '第{}台服务器名称:{},登录用户名:{},地址:{},端口:{}'.format(index + 1, sc.name, sc.username, sc.host, str(sc.port))
        if sc.secret_key_path is not None:
            config_str = config_str + ',密钥地址:{}'.format(sc.secret_key_path)
        config_str = config_str + '\n'
    config_str += "\n共{}台服务器".format(len(sc_list))
    click.echo(config_str)


def server_connect(name):
    click.echo("连接{}服务器...".format(name))
    sc = server_store.find_by_name(name)
    if sc is None:
        click.echo("未找到{}服务器配置!".format(name))
        return
    # 如果存在密钥 优先使用密钥登录服务器
    if sc.secret_key_path is not None:
        open_ssh_secret_key_tty(sc.host, sc.port, sc.username, sc.secret_key_path)
    else:
        open_ssh_password_tty(sc.host, sc.port, sc.username, sc.password)


def open_ssh_secret_key_tty(host, port, username, secret_key_path):
    cmd = 'ssh -o StrictHostKeyChecking=no -i {}  -p {} {}@{}'.format(secret_key_path, port, username, host)
    # 授权600
    os.chmod(secret_key_path, 0o600)
    p_ssh = pexpect.spawn(command=cmd)
    # 设置终端大小
    terminal_size = os.get_terminal_size()
    p_ssh.setwinsize(terminal_size.lines, terminal_size.columns)
    # 显示终端
    p_ssh.interact()


def open_ssh_password_tty(host, port, username, password):
    cmd = 'ssh -o StrictHostKeyChecking=no  -p {} {}@{}'.format(port, username, host)
    p_ssh = pexpect.spawn(command=cmd)
    # 输入密码
    try:
        p_ssh.expect("password:", timeout=10)
        p_ssh.sendline(password)
    except pexpect.exceptions.TIMEOUT:
        click.echo(click.style("等待输入密码消息超时!", fg='yellow'))
    # 设置终端大小
    terminal_size = os.get_terminal_size()
    p_ssh.setwinsize(terminal_size.lines, terminal_size.columns)
    # 显示终端
    p_ssh.interact()


def server_sftp(name, cwd_path):
    click.echo("连接{}服务器sftp服务...".format(name))
    sc = server_store.find_by_name(name)
    if sc is None:
        click.echo("未找到{}服务器配置!".format(name))
        return
    if sc.secret_key_path is not None:
        open_sftp_secret_key_tty(sc.host, sc.port, sc.username, sc.secret_key_path,
                                 cwd_path)
    else:
        open_sftp_password_tty(sc.host, sc.port, sc.username, sc.password, cwd_path)


def open_sftp_password_tty(host, port, username, password, cwd_path):
    # 执行sftp命令
    cmd = 'sftp -o StrictHostKeyChecking=no -P {} {}@{}'.format(port, username, host)
    p_sftp = pexpect.spawn(command=cmd, cwd=cwd_path)
    # 输入密码
    try:
        p_sftp.expect("password:", timeout=10)
        p_sftp.sendline(password)
    except pexpect.exceptions.TIMEOUT:
        click.echo(click.style("等待输入密码消息超时!", fg='yellow'))
    # 设置终端大小
    terminal_size = os.get_terminal_size()
    p_sftp.setwinsize(terminal_size.lines, terminal_size.columns)
    # 显示sftp终端
    p_sftp.interact()


def open_sftp_secret_key_tty(host, port, username, secret_key_path, cwd_path):
    # 执行sftp命令
    cmd = 'sftp -o StrictHostKeyChecking=no -i {} -P {} {}@{}'.format(secret_key_path, port, username, host)
    p_sftp = pexpect.spawn(command=cmd, cwd=cwd_path)
    # 设置终端大小
    terminal_size = os.get_terminal_size()
    p_sftp.setwinsize(terminal_size.lines, terminal_size.columns)
    # 显示sftp终端
    p_sftp.interact()


def ping_ssh_server(name):
    """
     检查 列表中的 server是否存活
    """
    sc = server_store.find_by_name(name)
    if sc is None:
        click.echo("未找到{}服务器配置!".format(name))
        return
    # 执行探测操作
    start_time = time.perf_counter_ns()
    r = open_socket_ssh_server(sc.host, sc.port)
    end_time = time.perf_counter_ns()
    result = "{},探测结果:{},耗时:{}ms".format(sc.host + ":" + str(sc.port), r,
                                         str(round((int(round((end_time - start_time) / 1000000))), 2)))
    click.echo(result)


def open_socket_ssh_server(host, port):
    try:
        remote_ip = host
        if compile_host_mame.match(host):
            remote_ip = socket.gethostbyname(host)
        elif compile_ip.match(host):
            remote_ip = socket.gethostbyaddr(host)
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        sock.connect((remote_ip, port))
        sock.close()
        return True
    except socket.error as e:
        click.echo("{}的{}端口连接失败,原因:{}".format(host, port, e))
        return False


def server_tun(left, right, reverse, name):
    sc = server_store.find_by_name(name)
    if sc is None:
        click.echo("未找到{}服务器配置!".format(name))
        return
    if reverse:
        cmd_str = 'ssh -CqTnN -R {} -p {} {}'.format('{}:{}'.format(get_server_config_tun_info(sc, right), left),
                                                     str(sc.port),
                                                     '{}@{}'.format(sc.username, sc.host))
        click.echo(cmd_str)
        auto_open_tun(cmd_str, sc.password)
    else:
        cmd_str = 'ssh -CqTnN -L {} -p {} {}'.format('{}:{}'.format(left, get_server_config_tun_info(sc, right)),
                                                     str(sc.port),
                                                     '{}@{}'.format(sc.username, sc.host))
        click.echo(cmd_str)
        auto_open_tun(cmd_str, sc.password)


def auto_open_tun(cmd_str, password):
    """
    自动打开 tun通道   当断开之后重连
    """
    p_tun = pexpect.spawn(command=cmd_str, cwd=".")
    # 输入密码
    try:
        p_tun.expect("password:", timeout=3)
        p_tun.sendline(password)
    except pexpect.exceptions.TIMEOUT:
        click.echo(click.style("等待输入密码消息超时!", fg='yellow'))
    pid = p_tun.pid
    click.echo(pid)
    while True:
        if not psutil.pid_exists(pid):
            p_tun = pexpect.spawn(command=cmd_str, cwd=".")
            # 输入密码
            try:
                p_tun.expect("password:", timeout=3)
                p_tun.sendline(password)
            except pexpect.exceptions.TIMEOUT:
                click.echo(click.style("等待输入密码消息超时!", fg='yellow'))
            pid = p_tun.pid
            click.echo("重连tun通道,PID:{}", pid)
        # 休眠200ms
        time.sleep(0.2)


def get_server_config_tun_info(sc: ServerConfig, tun_str):
    if compile_tun_value.match(tun_str):
        return tun_str
    else:
        return '{}:{}'.format(sc.host, sc.port)


def server_select_connect():
    # 获取服务器列表 增加编号
    sc_list = server_store.find_all()
    if sc_list is None:
        return
    config_str = '服务器信息:\n'
    index_name_map = {}
    for index, sc in enumerate(sc_list):
        index_name_map[str(index + 1)] = sc.name
        config_str += '{})台服务器名称:{},地址:{},端口:{}\n'.format(index + 1, sc.name, sc.host, str(sc.port))
    click.echo(config_str)
    # 等待用户输入服务器编号
    r = click.prompt(
        "选择服务器编号:",
        type=click.Choice(index_name_map.keys()),
        show_default=False,
    )
    server_connect(index_name_map[r])


def server_select_sftp(cwd):
    sc_list = server_store.find_all()
    if sc_list is None:
        return
    config_str = '服务器信息:\n'
    index_name_map = {}
    for index, sc in enumerate(sc_list):
        index_name_map[str(index + 1)] = sc.name
        config_str += '{})台服务器名称:{},地址:{},端口:{}\n'.format(index + 1, sc.name, sc.host, str(sc.port))
    click.echo(config_str)
    # 等待用户输入服务器编号
    r = click.prompt(
        "选择服务器编号:",
        type=click.Choice(index_name_map.keys()),
        show_default=False,
    )
    server_sftp(index_name_map[r], cwd)



import os
import re

# 版本号  setup.py  ming.py 引用  每次发布 版本+1
ming_global_version = 2.29


def get_version():
    return ming_global_version


class ToolsDependency:
    """
    工具依赖
    """
    cmd: str
    desc: str
    install_demo_cmd: str

    def __init__(self, cmd, desc=None, install_demo_cmd=None):
        self.cmd = cmd
        self.desc = desc
        self.install_demo_cmd = install_demo_cmd


# 工具依赖信息数组
tools_dependency_info_arr = [
    ToolsDependency('cat'),
    ToolsDependency('ssh'),
    ToolsDependency('sftp'),
    ToolsDependency('vi'),
    ToolsDependency('dd'),
    ToolsDependency('git'),
    ToolsDependency('free'),
    ToolsDependency('ip'),
    ToolsDependency('dot', "绘制svg图案,graphviz", 'yay -Syyu graphviz'),
    ToolsDependency('screenfetch', "显示当前操作系统信息", 'yay -Syyu screenfetch'),
    # ToolsDependency('sar', "显示当前操作系统信息", 'yay -Syyu screenfetch'),
    # ToolsDependency('showError', '测试错误情况', 'test'),
]

# 配置目录存放在 用户根目录
db_dir = '{}/.jiuming-tools/config/db'.format(os.path.expanduser('~'))
if not os.path.exists(db_dir):
    os.makedirs(db_dir)


# 配置目录存放在 用户根目录
root_config_dir = '{}/.jiuming-tools'.format(os.path.expanduser('~'))
if not os.path.exists(root_config_dir):
    os.makedirs(root_config_dir)

# 功能配置目录
config_default_file = '{}/.jiuming-tools/config'.format(os.path.expanduser('~'))
if not os.path.exists(config_default_file):
    os.makedirs(config_default_file)

# 脚本默认文件夹
script_default_file = '{}/.jiuming-tools/config/script'.format(os.path.expanduser('~'))
if not os.path.exists(script_default_file):
    os.makedirs(script_default_file)

# ssh 私钥默认文件夹
private_key_default_file = '{}/.jiuming-tools/config/privateKey'.format(os.path.expanduser('~'))
if not os.path.exists(private_key_default_file):
    os.makedirs(private_key_default_file)

# 正则
compile_ip = re.compile('^((25[0-5]|2[0-4]\d|[01]?\d\d?)\.){3}(25[0-5]|2[0-4]\d|[01]?\d\d?)$')
compile_host_mame = re.compile('^[a-zA-Z0-9][-a-zA-Z0-9]{0,62}(\.[a-zA-Z0-9][-a-zA-Z0-9]{0,62})+\.?$')
compile_tun_value = re.compile('^[a-zA-Z0-9]{0,62}:[a-zA-Z0-9]{0,62}$')





import asyncio
import json
import os
import re
import subprocess

import click
import psutil
from prettytable import PrettyTable

from src.utils.convertUtils import byte_length_format


async def async_get_screenfetch():
    # 尝试执行 screenfetch
    try:
        res = await asyncio.create_subprocess_shell("screenfetch", stdout=asyncio.subprocess.PIPE,
                                                    stderr=asyncio.subprocess.PIPE)
        sout, serr = await res.communicate()
        # res.returncode, sout, serr, res.pid
        if res.returncode == 0:
            return sout.decode('utf-8')
    except OSError:
        return '安装screenfetch可以获得screenfetch图！'


def echo_pc_info():
    """
    输出 设备产品信息
    :return:
    """
    os_info = os.uname()
    # 获取当前系统虚拟化方式
    virtual_type_split_arr = str.split(
        subprocess.getoutput("lscpu | grep -E  '超管理器厂商|Hypervisor vendor'").strip(''))
    virtual_type_str = '无'
    if len(virtual_type_split_arr) == 2:
        virtual_type_str = virtual_type_split_arr[1]
    system_str = '''
操作系统:
  用户名:     {}
  主机名:     {}
  发行版本:   {}
  内核版本:   {}
  硬件架构:   {}
  虚拟化方式:  {}    
'''.format(
        os.getenv("USER"),
        os_info.nodename,
        subprocess.getoutput('cat /etc/issue').strip('\n'),
        os_info.release,
        os_info.machine,
        virtual_type_str if virtual_type_str is not None else "无"
    )
    click.echo(system_str)

    cpu_str = '''cpu信息:
  cpu型号:    {}
  cpu物理核心: {}
  cpu逻辑核心: {}
'''.format(
        cpu_info().modelName,
        psutil.cpu_count(logical=False),
        psutil.cpu_count()
    )
    click.echo(cpu_str)
    cpu_load_str = '''cpu当前负载信息:
{}    
'''.format(subprocess.getoutput('mpstat -P ALL'))
    click.echo(cpu_load_str)

    memory_str = '''内存信息:
{}
'''.format(subprocess.getoutput('free -h'))
    click.echo(memory_str)

    net_str = '''网卡信息:
{}    
'''.format(subprocess.getoutput('ip addr'))
    click.echo(net_str)

    disk_str = '''磁盘信息:
{}
'''.format(subprocess.getoutput('df -h'))
    click.echo(disk_str)

    inode_str = '''inode使用信息:
{}    
'''.format(subprocess.getoutput('df -ih'))
    click.echo(inode_str)

    ss_str = '''socket使用摘要统计信息:
{}    
'''.format(subprocess.getoutput('ss -s '))
    click.echo(ss_str)

    # 异步执行 screenfetch
    loop = asyncio.get_event_loop()
    screenfetch_future = asyncio.ensure_future(async_get_screenfetch(), loop=loop)
    loop.run_until_complete(screenfetch_future)
    screenfetch_result = screenfetch_future.result()
    if screenfetch_future is not None:
        screenfetch_str = '''screenfetch:
        \r\n
        {}
        '''.format(str(screenfetch_result))
        click.echo(screenfetch_str)


def cpu_info():
    """
    从 /proc/cpuinfo 中读取cpu 模型名称
    :return:
    """
    f_cpu_info = open('/proc/cpuinfo', 'r')
    c = f_cpu_info.readlines()
    for i in c:
        tmp_str_list = i.split(':')
        if re.match(".*model.*name.*", tmp_str_list[0]):
            return cpuInfo(tmp_str_list[1].strip('\n').strip('\t'))


class cpuInfo(object):
    def __init__(self, modelName):
        self.modelName = modelName


def pid_info(pid, details):
    """
    使用psutil 探测指定pid的综合信息 方便判断问题节点
    https://psutil.readthedocs.io/en/latest/#process-class
    获取对应id的进程综合信息
    cpu、内存、磁盘、文件描述符等等信息
    :return:
    """
    p = psutil.Process(pid)
    file_mem_map_info_str = ""
    for m in p.memory_maps():
        file_mem_map_info_str = file_mem_map_info_str + "\r\n" + m[0] + "使用rss:{},size:{}".format(
            byte_length_format(m[1]),
            byte_length_format(m[2]))

    mem_full_info = p.memory_full_info()
    mem_full_info_str = "使用总内存大小{},swap大小{}".format(byte_length_format(mem_full_info[8]),
                                                             byte_length_format(mem_full_info[9]))

    base_str = """
PID:{}
name:{}    
执行用户:{}
进程可执行文件路径:{}
进程使用的内存大小信息:{}
内存引用文件占用内存信息:{}
此进程当前打开的文件描述符数:{}
此进程当前使用的线程数:{}
此进程和总物理内存比例:{}
进程打开的套接字连接:{}
""".format(p.pid,
           p.name(),
           p.username(),
           p.exe(),
           mem_full_info_str,
           file_mem_map_info_str,
           p.num_fds(),
           p.num_threads(),
           p.memory_percent(),
           p.connections()
           )
    click.echo(base_str)
    if details:
        click.echo("详细信息:\r\n" + json.dumps(p.as_dict(), indent=1, separators=(', ', ': '), ensure_ascii=False))


def detect():
    """
    探测服务器各项资源  
    all:所有资源
    cpu:cpu资源
    mem:内存资源
    inode:inode资源
    net:网络带宽、和端口、当前网络连接资源  
    file_desc:文件描述符资源  
    io:io资源  
    
    """
    detect_cpu()
    detect_mem()


def detect_cpu():
    cpu_load_arr = psutil.cpu_percent(3, True)
    click.echo("cpu负载信息:")
    for i, c in enumerate(cpu_load_arr):
        click.echo("第{}个cpu负载:{}".format(i, c))
    click.echo("总负载:{}".format(sum(cpu_load_arr)))


def detect_mem():
    v_mem = psutil.virtual_memory()
    s_mem = psutil.swap_memory()
    mem_info = """
    物理内存:{}/{},{}
    虚拟内存:{}/{},{}
    """.format(byte_length_format(v_mem.used()), byte_length_format(v_mem.total()), 0, byte_length_format(s_mem.used),
               byte_length_format(s_mem.total), 0)
    click.echo(mem_info)


def detect_inode():
    pass


def detect_net():
    pass


def detect_file_desc():
    pass


def detect_io():
    pass


def mem_info(top, pid, details):
    p_arr = []
    if pid is not None:
        p_arr.append(psutil.Process(pid))
    else:
        for p in psutil.process_iter():
            p_arr.append(p)

    mem_info_arr = []
    for p in p_arr:
        for m in p.memory_maps():
            mem_info_arr.append(m)

    # 过滤 anon  heap  stack 数据
    mem_info_arr = [t for t in mem_info_arr if not (t[0] == '[anon]' or t[0] == '[heap]' or t[0] == '[stack]')]
    mem_info_arr.sort(key=lambda t: t[2], reverse=True)
    mem_info_arr = mem_info_arr[:top]

    # 创建表格
    t_table = PrettyTable(['file', 'rss', 'size'])
    for i, m in enumerate(mem_info_arr):
        t_table.add_row([m[0], byte_length_format(m[1]), byte_length_format(m[2])])
        if details:
            click.echo("详细信息:\r\n" + json.dumps(m, indent=1, separators=(', ', ': '), ensure_ascii=False))
    # 设置对齐方式
    t_table.align = "l"
    # 设置表格最长列
    t_table.max_width = 80
    click.echo(t_table)


def net_info(details):
    all_net_io_counter = psutil.net_io_counters()
    all_net_io_info_str = """
综合数据:     
发送数据大小: {}
接受数据大小: {}
发送的数据包数量: {}
接受的数据包数量: {}
接受的错误总数: {}
发送的错误总数: {}
丢弃的传入数据包总数: {}
丢弃的传出数据包总数: {}
    """.format(byte_length_format(all_net_io_counter.bytes_sent),
               byte_length_format(all_net_io_counter.bytes_recv),
               all_net_io_counter.packets_sent,
               all_net_io_counter.packets_recv,
               all_net_io_counter.errin,
               all_net_io_counter.errout,
               all_net_io_counter.dropin,
               all_net_io_counter.dropout
               )
    print(all_net_io_info_str)

    net_io_counters = psutil.net_io_counters(pernic=True)

    net_io_table = PrettyTable(
        ['网络接口名称', '发送数据大小', '接受数据大小', '发送的数据包数量', '接受的数据包数量', '接受的错误总数',
         '发送的错误总数',
         '丢弃的传入数据包总数',
         '丢弃的传出数据包总数'])
    for key in net_io_counters.keys():
        t = net_io_counters[key]
        net_io_table.add_row([key,
                              byte_length_format(t.bytes_sent),
                              byte_length_format(t.bytes_recv),
                              t.packets_sent,
                              t.packets_recv,
                              t.errin,
                              t.errout,
                              t.dropin,
                              t.dropout
                              ])

    net_io_table.align = "l"
    # 设置表格最长列
    net_io_table.max_width = 80
    click.echo(net_io_table)
    pass


def disk_info():
    disk_partitions = psutil.disk_partitions()
    d_p_table = PrettyTable(['设备驱动号', '挂载名称', '文件系统类型', '挂载选项', '使用量'])
    for d in disk_partitions:
        t_l = [d.device, d.mountpoint, d.fstype, d.opts]
        t_d_u = psutil.disk_usage(d.mountpoint)
        t_l.append("total:{},used:{},free:{},percent:{}%".format(byte_length_format(t_d_u.total),
                                                                 byte_length_format(t_d_u.used),
                                                                 byte_length_format(t_d_u.free),
                                                                 t_d_u.percent
                                                                 ))
        d_p_table.add_row(t_l)
    # 设置对齐方式
    d_p_table.align = "l"
    # 设置表格最长列
    d_p_table.max_width = 80
    click.echo(d_p_table)

    # 输出当前磁盘的详细io信息
    try:
        disk_stat_dict = psutil.disk_io_counters(perdisk=True)
        d_s_table = PrettyTable(
            ['设备名', '读取次数', '写入次数', '读取大小', '写入大小', '读取耗时(ms)', '写入耗时(ms)',
             '实际io耗费时间(ms)', '合并读取次数', '合并写入次数'])
        for key in disk_stat_dict.keys():
            t = disk_stat_dict[key]
            d_s_table.add_row([key,
                               t.read_count,
                               t.write_count,
                               byte_length_format(t.read_bytes),
                               byte_length_format(t.write_bytes),
                               t.read_time,
                               t.write_time,
                               t.busy_time,
                               t.read_merged_count,
                               t.write_merged_count
                               ])

        d_s_table.align = "l"
        # 设置表格最长列
        d_s_table.max_width = 80
        click.echo(d_s_table)
    except Exception as e:
        click.echo(click.style("获取磁盘io统计计数信息异常!" + e, fg='red'))



def byte_length_format(byte_length):
    units = ["B", "KB", "MB", "GB", "TB", "PB"]
    size = 1024.0
    for i in range(len(units)):
        if (byte_length / size) < 1:
            return "%.2f%s" % (byte_length, units[i])
        byte_length = byte_length / size





import click
from lxml import etree
from requests_cache import CachedSession

def search(name):
    # 处理空格
    name = name.strip()
    # 增加缓存处理
    session = CachedSession('jiuming-tools_http_cache', backend='filesystem', use_cache_dir=True)
    response = session.get("http://linux.51yip.com/search/{}".format(name))
    response.encoding = 'utf-8'
    if response.status_code != 200:
        click.echo(response.text)  # 以文本形式打印网页源码
        if response.status_code == 404:
            click.echo("未找到{}命令!".format(name))
        if response.status_code >= 500:
            click.echo("51yip服务器异常!")
        return
    # 利用etree.HTML，将字符串解析为HTML文档
    html = etree.HTML(response.text, etree.HTMLParser())
    click.echo(html.xpath('string(//*[@id="post-1"]/h1)'))
    click.echo(html.xpath('string(//*[@id="post-1"]/div[1])'))
    content_arr = html.xpath('//*[@id="post-1"]/div[2]/pre')
    for i, content in enumerate(content_arr):
        if i == 0:
            help_arr = content.xpath('text()')
            for h in help_arr:
                click.echo(h)
        else:
            click.echo(content.xpath('string(.)'))



import copy
import os
import pathlib
import stat
import time

import click
import yaml

from src.config.global_config import config_default_file, script_default_file

script_config_default_file = config_default_file + '/script_config.yaml'


def script_create(type, name, remark):
    """
        添加服务配置
        :param type: 脚本类型名称
        :param name: 服务器名称
        :param remark: 脚本备注
        :return:
        """
    # 追加模式
    s_file_path = script_default_file + '/{}'.format(name)
    if os.path.exists(s_file_path):
        click.echo("{}已存在!".format(name))
        return
    y_file = open(script_config_default_file, 'a+')
    sc = ScriptConfig(type=type, name=name, remark=remark, path=s_file_path)
    yaml.safe_dump([sc.__dict__], y_file)
    s_file = open(s_file_path, 'w+')
    write_str = """#!/usr/bin/env {} 
#author ming 
#创建时间:{} 
#{}
""".format(type, time.strftime('%Y-%m-%d %H:%M:%S'), remark)
    s_file.write(write_str)
    s_file.close()
    os.chmod(s_file_path, stat.S_IRWXU)
    os.system('vi {}'.format(s_file_path))


def script_remove(name):
    """
    删除服务器配置信息
    获取所有配置  删除其中name符合的数据
    :param name: 服务器名称
    :return:
    """
    y_read_file = open(script_config_default_file, 'r')
    config_list = yaml.safe_load(y_read_file)
    if config_list is None:
        click.echo("暂无脚本!")
        return
    # 深拷贝 配置列表 进行操作列表
    new_config_list = copy.deepcopy(config_list)
    for c in config_list:
        sc = ScriptConfig.to_obj(c)
        if sc.name == name:
            new_config_list.remove(c)
            os.remove(sc.path)
            click.echo("删除{}脚本".format(sc.name))
    # 重新打开链接
    if len(new_config_list) != 0:
        yaml.safe_dump(new_config_list, open(script_config_default_file, 'w+'))
    else:
        # 清空配置
        open(script_config_default_file, 'w+').truncate()
        click.echo("脚本已清空!")


def script_list():
    config_file = pathlib.Path(script_config_default_file)
    if not config_file.exists():
        click.echo("暂无脚本！")
        return
    if not config_file.is_file():
        click.echo("{}不是配置文件".format(script_config_default_file))
        return
    y_read_file = open(script_config_default_file, 'r')
    config_list = yaml.safe_load(y_read_file)
    if config_list is None:
        click.echo("暂无脚本!")
        return
    config_str = '脚本列表信息:\n'
    for index, c in enumerate(config_list):
        sc = ScriptConfig.to_obj(c)
        config_str += '第{}个脚本名称:{},执行引擎:{},备注:{},脚本地址:{}'.format(index + 1, sc.name, sc.type, sc.remark, sc.path) + '\n'
    config_str += "\n共{}脚本\n".format(len(config_list))
    click.echo(config_str)


def script_details(name):
    y_read_file = open(script_config_default_file, 'r')
    config_list = yaml.safe_load(y_read_file)
    if config_list is None:
        click.echo("暂无脚本!")
        return
    for c in config_list:
        sc = ScriptConfig.to_obj(c)
        if sc.name == name:
            os.system('cat {}'.format(sc.path))


def script_exec(name):
    y_read_file = open(script_config_default_file, 'r')
    config_list = yaml.safe_load(y_read_file)
    if config_list is None:
        click.echo("暂无脚本!")
        return
    for c in config_list:
        sc = ScriptConfig.to_obj(c)
        if sc.name == name:
            start_time = time.perf_counter_ns()
            click.echo(
                click.style("{}脚本开始执行,脚本引擎{},脚本备注:{},脚本地址:{}".format(sc.name, sc.type, sc.remark, sc.path), fg='green'))
            click.echo(click.style("start--------------------------------", fg='green'))
            result = os.system(sc.path)
            end_time = time.perf_counter_ns()
            click.echo(click.style("end--------------------------------", fg='green'))
            click.echo("{}执行耗时:{}ms,结果:{}".format(sc.name, str(
                round((int(round((end_time - start_time) / 1000000))), 2)), str(result)))


class ScriptConfig(object):
    """
    脚本配置 对象
    """
    # 脚本类型  如 sh  bash  python   以linux shell的脚本引擎名称   执行脚本的时候  就是 type ./xxx
    type: str
    # 脚本名称
    name: str
    # 脚本备注
    remark: str
    # 脚本地址
    path: str

    def __init__(self, type, name, remark, path):
        self.type = type
        self.name = name
        self.remark = remark
        self.path = path

    @staticmethod
    def to_obj(d: dict):
        """
        将读取的dict 转换为 serverConfig
        :param d:  dict
        :return: ServerConfig
        """
        return ScriptConfig(type=d['type'], name=d['name'],
                            remark=d['remark'], path=d['path'])



# -*- coding:utf-8 -*-
from setuptools import setup, find_packages

from src.config.global_config import get_version

setup(
    name='jiuming-tools',
    # 版本 如果需要发布更新 需要调整版本号
    version=get_version(),
    packages=find_packages(),
    include_package_data=True,
    # data_files=[
    #     # 打包打进ming.yaml
    #     ('', ['ming.yaml'])
    # ],
    platforms='linux',
    # 需要的依赖
    install_requires=[
        'Click',
        'psutil',
        'PyYAML',
        'gitpython',
        'bcrypt',
        'speedtest-cli',
        'pexpect',
        'scapy',
        'lxml',
        'requests-cache',
        'peewee',
        'prettytable'
    ],
    # 命令行入口
    entry_points='''
        [console_scripts]
        m=src.ming:cli
    ''',
    author='ming',
    license='MIT',
    url='https://github.com/xujiuming/jiuming-tools',
    description='个人常用功能'
)



from peewee import *

# 创建sqlite
from src.config.global_config import db_dir

db = SqliteDatabase(db_dir + '/tools.db')


# 定义peewee 元数据
class BaseModel(Model):
    class Meta:
        database = db


class MyTask(BaseModel):
    name = CharField()
    content = TextField()
    level = IntegerField()
    over = BooleanField()
    over_time = TimestampField()
    create_time = TimestampField()
    update_time = TimestampField()


db.connect()
db.create_tables([MyTask])



import copy
import os
import pathlib

import click
import yaml

from src.config.global_config import config_default_file

# 默认配置file
server_config_default_file = config_default_file + '/server_config.yaml'


def create(sc):
    y_file = open(server_config_default_file, 'a+')
    yaml.safe_dump([sc.__dict__], y_file)


def edit():
    click.edit(filename=server_config_default_file)


def remove_by_name(name):
    name = name.strip()
    sc_list = find_all()
    if sc_list is None:
        return None
    # 深拷贝 配置列表 进行操作列表
    new_sc_list = copy.deepcopy(sc_list)
    for index, sc in enumerate(sc_list):
        if sc.name == name:
            new_sc_list.remove(new_sc_list[index])
            if sc.secret_key_path is not None:
                os.remove(sc.secret_key_path)
            click.echo("删除{}服务器".format(sc.name))
    # 重新打开链接
    if len(new_sc_list) != 0:
        dict_list = []
        for sc in new_sc_list:
            dict_list.append(sc.__dict__)
        yaml.safe_dump(dict_list, open(server_config_default_file, 'w+'))
    else:
        # 清空配置
        open(server_config_default_file, 'w+').truncate()
        click.echo("服务器配置已清空!")


def find_by_name(name):
    name = name.strip()
    sc_list = find_all()
    if sc_list is None:
        return None
    for sc in sc_list:
        if name == sc.name:
            return sc
    return None


def find_all():
    config_file = pathlib.Path(server_config_default_file)
    if not config_file.exists():
        click.echo("暂无服务器配置信息！")
        return None
    if not config_file.is_file():
        click.echo("{}不是配置文件".format(server_config_default_file))
        return None
    config_list = yaml.safe_load(open(server_config_default_file, 'r'))
    if config_list is None:
        click.echo("暂无服务器配置信息!")
        return None
    result = []
    for c in config_list:
        result.append(ServerConfig.to_obj(c))
    return result


class ServerConfig(object):
    """
    服务器配置模板class
    """
    # 名字
    name: str
    # 地址
    host: str
    # ssh端口
    port: int
    # 服务器用户名
    username: str
    # 密码
    password: str
    # 密钥位置
    secret_key_path: str

    def __init__(self, name, host, port, username, password, secret_key_path):
        self.name = name
        self.host = host
        self.port = port
        self.username = username
        self.password = password
        self.secret_key_path = secret_key_path

    @staticmethod
    def to_obj(d: dict):
        """
        将读取的dict 转换为 serverConfig
        :param d:  dict
        :return: ServerConfig
        """
        return ServerConfig(name=d['name'], host=d['host'], port=d['port'], username=d['username'],
                            password=d['password'], secret_key_path=d.get('secret_key_path', None))




class ScriptConfig(object):
    """
    脚本配置 对象
    """
    # 脚本类型  如 sh  bash  python   以linux shell的脚本引擎名称   执行脚本的时候  就是 type ./xxx
    type: str
    # 脚本名称
    name: str
    # 脚本备注
    remark: str
    # 脚本地址
    path: str

    def __init__(self, type, name, remark, path):
        self.type = type
        self.name = name
        self.remark = remark
        self.path = path

    @staticmethod
    def to_obj(d: dict):
        """
        将读取的dict 转换为 serverConfig
        :param d:  dict
        :return: ServerConfig
        """
        return ScriptConfig(type=d['type'], name=d['name'],
                            remark=d['remark'], path=d['path'])
